require 'colorize'
require 'terminal-table'

class Drone::Builds::Parser
  attr_reader :client

  STATUS_COLORS = {
    success: :light_green,
    running: :light_blue,
    failure: :light_red,
    killed: :light_white,
    pending: :light_yellow
  }.freeze

  MESSAGE_MAX_WIDTH = 45

  def initialize(client)
    @client = client
  end

  def parse_builds(builds)
    builds.map { |build| parse_build(build) }
  end

  def parse_build(build)
    {
      number: parse_number(build['number']),
      status: parse_status(build['status']),
      author: parse_author(build['author']),
      ref: parse_ref(build['ref']),
      message: parse_message(build['message'])
    }
  end

  def parse_number(number)
    number.to_s
  end

  def parse_status(status)
    status.colorize(STATUS_COLORS[status.to_sym])
  end

  def parse_author(author)
    current_github_user = HOME_ENV[:GITHUB_USER]
    if author == current_github_user
      author.colorize(:light_white)
    else
      author
    end
  end

  def parse_ref(ref)
    ref.gsub('refs/heads/', '').gsub('/head', '').gsub('refs/pull/', '#')
  end

  def parse_message(message)
    message = message.to_s.lines.first.to_s
    message = message.gsub("of github.com:#{client.owner}/#{client.name}", '')
    message[0..MESSAGE_MAX_WIDTH]
  end

  def table(data, fields)
    Terminal::Table.new(rows: [fields, :separator] + data.map(&:values))
  end
end
